
#import "MP3List.h"
#import <stdlib.h>			// for free() etc
#import <strings.h>			// for strcasecmp()
#import <objc/hashtable.h>		// for NXCopyStringBuffer()

#define str_copy(str)	((str == NULL) ? NULL : NXCopyStringBuffer(str))
#define str_free(str)	{if (str) free(str);}

@implementation MP3Info
- init {
	return [self initWithFilename:NULL path:NULL];
}
- initWithFilename:(const char *)name path:(const char *)pathstr {
	[super init];
	filename = str_copy(name);
	path = str_copy(pathstr);
	return self;
}

- setFileName:(const char *)value {
	str_free(filename);
	filename = str_copy(value);
	return self;
}
- (const char *)filename {
	return filename;
}
- setPath:(const char *)value {
	str_free(path);
	path = str_copy(value);
	return self;
}
- (const char *)path {
	return path;
}

- read:(NXTypedStream *)stream {
	int length;
	char buffer[256];
	
	[super read:stream];
	NXReadType(stream, "i", &length);
	NXReadArray(stream, "c", length, buffer);
	buffer[length] = '\0';
	[self setPath:buffer];
	NXReadType(stream, "i", &length);
	NXReadArray(stream, "c", length, buffer);
	buffer[length] = '\0';
	[self setFileName:buffer];
	
	return self;
}
- write:(NXTypedStream *)stream {
	int length; 
	[super write:stream];
	
	length = strlen(path);
	NXWriteType(stream, "i", &length);
	NXWriteArray(stream, "c", length, path);

	length = strlen(filename);
	NXWriteType(stream, "i", &length);
	NXWriteArray(stream, "c", length, filename);
	return self;
}

- copy { // by hrjun 06.03
	MP3Info *copiedObj = [[MP3Info alloc] initWithFilename:filename path:path];
	return copiedObj;
}

- free {
	str_free(filename);
	str_free(path);
	return [super free];
}
@end


@implementation MP3List
static int docompare(const void *x, const void *y)
{
	return strcasecmp([(id)(*(MP3Info **)x) filename], [(id)(*(MP3Info **)y) filename]);
}

- sort {
	qsort((MP3Info **)dataPtr, numElements, sizeof(id), docompare);
	return self;
}

// by hrjun 06.03
- moveObjectFrom:(unsigned)from to:(unsigned)to {
	MP3Info *newOne = nil;
	if (from == to) return nil;
	else {
		MP3Info *oldOne = [self objectAt:from];
		newOne = [oldOne copy];
		[self insertObject:newOne at:to];
		[self removeObject:oldOne];
		[oldOne free];
	}
	return newOne;
}
@end
