#import "MP3ListManager.h"
#import "MP3List.h"
#import "MainControl.h"
#import "MP3Process.h"
#import "Common.h"
#import <sys/time.h>
#import <string.h>

/* (by capri91) */
static const char *const mp3Type[3] = {"mp3", "MP3", NULL};
static const char *const docType[2] = {DOC_EXTENSION, NULL};

@interface MP3ListManager (Private)
- writeList:(const char *)filename;
@end

@implementation MP3ListManager

+ initialize {
    struct timeval tv;
    struct timezone tz;

    gettimeofday(&tv, &tz);
    srandom(tv.tv_sec);
    return self;
}

- init {
    [super init];
    [NXApp loadNibSection:"MP3ListManager.nib" owner:self];

    fileList = [[MP3List alloc] init];
    playList = [[MP3List alloc] init];
    return self;
}

- free {
    [fileList freeObjects];
    [fileList free];
    [playList free];
    [panel free];
    return [super free];
}

- awakeFromNib {
    NXRect rect;
    NXSize size;

    [browser setMaxVisibleColumns:1];
    [browser setHorizontalScrollButtonsEnabled:NO];
    [browser setHorizontalScrollerEnabled:NO];
    [browser separateColumns:NO];
    [browser setTitled:NO];
    [browser setMultipleSelectionEnabled:YES];
    [browser setBranchSelectionEnabled:YES];
    [browser setEmptySelectionEnabled:YES];
    [browser acceptArrowKeys:YES andSendActionMessages:YES];
    [browser setTarget:self];
    [browser setDoubleAction:@selector(playSelection:)];

//    [panel setMiniwindowIcon:"Sound.tiff"];
//    [panel setMiniwindowTitle:"MP3List"];
    [panel getFrame:&rect];
    size.width = rect.size.width * 2;
    size.height = rect.size.height * 3;
    [panel setMaxSize:&size];

    return self;
}

- (BOOL)isRepeat {
    return isRepeat;
}

- isRepeat:(BOOL)yesNo {
    isRepeat = yesNo;
    return self;
}

- (BOOL)isShuffle {
    return isShuffle;
}

- isShuffle:(BOOL)yesNo {
    isShuffle = yesNo;
    return self;
}

- reset {
    unsigned int count;

    [playList empty];
    [playList appendList:fileList];
    count = [playList count];

    currentIndex = 0;
    if (count && isShuffle)
        currentIndex = (float)random() / LONG_MAX * (count - 1);

    return self;
}

- previous {
    if (currentIndex)
        currentIndex--;
    else if (isRepeat)
        currentIndex = [fileList count] - 1;
    [[browser matrixInColumn:0] selectCellAt:currentIndex :0]; // by hrjun 06.03

    return self;
}

- next {
    if (currentIndex < ([fileList count] - 1))
        currentIndex++;
    else if (isRepeat)
        currentIndex = 0;
    [[browser matrixInColumn:0] selectCellAt:currentIndex :0]; // by hrjun 06.03

    return self;
}

/*- follow {
    unsigned int count;
    unsigned int oldIndex = currentIndex;

    count = [playList count];
//    printf("follow.count: %d, index:%d\n", count, currentIndex); //***
    if (count > 0) {
        if (isShuffle) currentIndex = (float)random() / LONG_MAX * (count - 2);
        else {
//            printf("follow.next: %d\n"); //***
            [self next];
            [playList removeObject:[fileList objectAt:oldIndex]];
        }
    } else {
        if (isRepeat) {
            [self reset];
//            printf("follow.repeat.reset: %d\n", [playList count]); //***
            return self;
        } else {
//            printf("follow.norepeat.reset\n"); //***
            [playList removeObject:[fileList objectAt:oldIndex]];
            [[browser matrixInColumn:0] selectCellAt:0 :0];
            return self;
        }
    }

    printf("follow.count: %d, index:%d\n", [playList count], currentIndex); //***
    [[browser matrixInColumn:0] selectCellAt:currentIndex :0];

    return self;
}*/

- follow {
    unsigned int count;

    [playList removeObject:[fileList objectAt:currentIndex]];
    if (isShuffle) {
        count = [playList count];
        if (!count && isRepeat)
            [self reset];
        else if (count)
            currentIndex = [fileList indexOf:[playList objectAt:((float)random() / LONG_MAX * (count - 1))]];
    }
    else
        [self next];

    return self;
}

- currentFile {
    if ([fileList count])
        return [fileList objectAt:currentIndex];
    else
        return nil;
}

- (BOOL)loadMP3List:(const char *)filename {
    id list = nil;
    NXStream *stream = NXMapFile(filename, NX_READONLY);
    NXTypedStream *ts = NULL;

    if (stream) {
        NX_DURING
            ts = NXOpenTypedStream(stream, NX_READONLY);
            if (ts == NULL) return NO;
            list = NXReadObject(ts);
        NX_HANDLER
            printf("Fail to open file...%s\n", filename);
            return NO;
        NX_ENDHANDLER

        NXCloseTypedStream(ts);
        NXCloseMemory(stream, NX_FREEBUFFER);
    }
    if (list) {
        [self reset];
        [playList empty];
        [fileList empty];
        [fileList appendList:list];
        [browser loadColumnZero];
        [[browser matrixInColumn:0] selectAll:nil];
    }
    return YES;
}

- (BOOL)addToPlayList:(char *)file {
    int i;
    char *ptr = strrchr(file, '.');

    for (i = 0; mp3Type[i]; i++)
        if (!strcmp(ptr + 1, mp3Type[i]))
            break;

    if (mp3Type[i]) {
        MP3Info *info;
        ptr = strrchr(file, '/');
        if (ptr) {
            *ptr = '\0';
            info = [[MP3Info alloc] initWithFilename:ptr + 1 path:file];
            [fileList addObject:info];
            *ptr = '/';
        }
        else {
            info = [[MP3Info alloc] initWithFilename:file path:NULL];
            [fileList addObject:info];
        }
        return YES;
    }
    return NO;
}

- (BOOL)addMP3Files:(char *)files {
    char *ptr = files;
    char *tab;
    BOOL success = NO;

    while (tab = strchr(ptr, '\t')) {
        *tab = '\0';
	success = ([self addToPlayList:ptr] || success);
	*tab = '\t';
	ptr = tab + 1;
    }
    if (*ptr)
        success = ([self addToPlayList:ptr] || success);

    if (success) {
        [self reset];
        [browser loadColumnZero];
//        [[browser matrixInColumn:0] selectAll:self];
    }
    return success;
}

- open:sender {
    id open = [OpenPanel new];

    [open allowMultipleFiles:NO];
    [open chooseDirectories:NO];
    [open setTitle:"Open MP3 List..."];
    if ([open runModalForTypes:docType]) {
        if ([self loadMP3List:[open filename]])
            [[NXApp delegate] tryToAddPlayList:[open filename]];
    }
    return self;
}

- save:sender { // changed by hrjun 06.03
    if ([fileList count] < 1) return nil;
    if ([[NXApp delegate] currentListName]) {
        [self writeList:[[NXApp delegate] currentListName]];
    } else {
        id save = [SavePanel new];

        [save setRequiredFileType:DOC_EXTENSION];
        [save setTitle:"Save MP3 List..."];
        if ([save runModalForDirectory:NXHomeDirectory() file:NULL]) {
            [self writeList:[save filename]];
        }
    }
    return self;
}

- saveAs:sender { // by hrjun 06.03
    id save = [SavePanel new];

    if ([fileList count] < 1) return nil;
    [save setRequiredFileType:DOC_EXTENSION];
    [save setTitle:"SaveAs MP3 List..."];
    if ([save runModalForDirectory:NXHomeDirectory() file:NULL]) {
        [self writeList:[save filename]];
    }

    return self;
}

- addFile:sender {
    id open = [OpenPanel new];

    [open allowMultipleFiles:YES];
    [open chooseDirectories:NO];
    [open setTitle:"Add MP3..."];

    if ([open runModalForTypes:mp3Type]) {
        id matrix = nil;
        const char *const *files = [open filenames];
        for (; files && *files; files++) {
            MP3Info *info = [[MP3Info alloc] initWithFilename:*files path:[open directory]];
            [fileList addObject:info];
        }
        [self reset];
        [browser loadColumnZero];
        matrix = [browser matrixInColumn:0];
        [matrix selectCellAt:[matrix cellCount]-1 :0];
//        [[browser matrixInColumn:0] selectAll:sender];
    }
    return self;
}

- removeFile:sender { // changed by hrjun 06.03
    List *list = [[List alloc] init];
    id matrix = [browser matrixInColumn:0];
    int i, lastIndex=0;

    [matrix getSelectedCells:list];
    if ([list count] < 1) return nil;
    for (i = [list count] - 1; i >= 0; i--) {
        int row, col;
        MP3Info *file = nil;
        [matrix getRow:&row andCol:&col ofCell:[list objectAt:i]];
        if (i == [list count]-1) lastIndex = row;
        file = [fileList objectAt:row];
        [playList removeObject:file];
        [fileList removeObjectAt:row];
    }
    [browser loadColumnZero];
    matrix = [browser matrixInColumn:0];
    if (lastIndex < [matrix cellCount]-1) [matrix selectCellAt:lastIndex :0];
    else [matrix  selectCellAt:[matrix cellCount]-1 :0];
    [list free];
    return self;
}

- selectAll:sender {
    id matrix = [browser matrixInColumn:0];
    [matrix selectAll:sender];
    return self;
}

- playSelection:sender {
    int row = [[browser matrixInColumn:0] selectedRow];

    if (row > -1) {
        MP3Process *process = [MP3Process sharedProcess];

        if ([process status] == PS_STOPPED)
            [self reset];

        currentIndex = row;
        [process stop];		
        [[NXApp delegate] startPlay];
    }
    return self;
}

// by hrjun 06.03
- moveList:sender {
    int i;
    List *list = NULL;
    id matrix = [browser matrixInColumn:0];
    int total = [[matrix cellList] count];

    if (total < 1) return nil;

    list = [[List alloc] init];	
    [matrix getSelectedCells:list];
    for (i = [list count] - 1; i >= 0; i--) {
//        MP3Info *file = nil;
        int row, col;
        [matrix getRow:&row andCol:&col ofCell:[list objectAt:i]];
        switch([[sender selectedCell] tag]) {
            case 0: // up
                if (row == 0) continue;
                else {
                    id file = [playList objectAt:row];
                    if ([playList indexOf:file] != NX_NOT_IN_LIST) {
                        [playList removeObject:file];
                        file = [fileList moveObjectFrom:row to:row-1];
                        [playList addObject:file];
                    } else {
                        [fileList moveObjectFrom:row to:row-1];
                    }
                    [browser loadColumnZero];
                    matrix = [browser matrixInColumn:0];
                    [matrix  selectCellAt:row-1 :col];
                }
                break;
            case 1: // down
                if (row == total-1) continue;
                else {
                    id file = [playList objectAt:row];
                    if ([playList indexOf:file] != NX_NOT_IN_LIST) {
                        [playList removeObject:file];
                        file = [fileList moveObjectFrom:row to:row+2];
                        [playList addObject:file];
                    } else {
                        [fileList moveObjectFrom:row to:row+2];
                    }
                    [browser loadColumnZero];
                    matrix = [browser matrixInColumn:0];
                    [matrix selectCellAt:row+1 :col];
                }
                break;
        }
    }
    [list free];
    return self;
}

- showPanel {
    [panel orderFront:self];
    return self;
}

- hidePanel {
    [panel orderOut:self];
    return self;
}

- movePanelTopLeftTo:(NXCoord)x :(NXCoord)y {
    [panel moveTopLeftTo:x :y];
    return self;
}

@end

@implementation MP3ListManager (Private)

- writeList:(const char *)filename {
    int cnt;
    NXTypedStream *ts;

    cnt = [fileList count];
    if (cnt < 1) return nil;
    ts = NXOpenTypedStreamForFile(filename, NX_WRITEONLY);
    if (ts == NULL) {
        NXRunAlertPanel("Alert!", "Fail to open a streram of '%s'...", 0, 0, 0, filename);
        return nil;
    }
    NXWriteObject(ts, fileList);
    NXCloseTypedStream(ts);

    return self;
}

@end

@implementation MP3ListManager (BrowserDelegate)

- addCellWithString:(const char *)str at:(int)row matrix:matrix {
    id cell;

    [matrix insertRowAt:row];
    cell = [matrix cellAt:row :0];
    [cell setStringValue:str];
    [cell setLoaded:YES];
    [cell setLeaf:YES];
    [cell setBordered:YES];
/* (by capri91) */
    [cell setFont:[Font userFontOfSize:10 matrix:NX_FLIPPEDMATRIX]];
    return self;
}

- (int)browser:sender fillMatrix:matrix inColumn:(int)column {
    if (column == 0) {
        int i, cnt = [fileList count];
        for (i = 0; i < cnt; i++) {
            char buffer[MAXPATHLEN];
            id info = [fileList objectAt:i];
            sprintf(buffer, "%s/%s", [info path], [info filename]);
            [self addCellWithString:[info filename] at:i matrix:matrix];
        }
        return [matrix cellCount];
    }
    return 0;
}

@end

@implementation MP3ListManager (WindowDelegate)

- windowWillClose:sender {
    SEL aSelector = @selector(panelWillClose:);
    if ([[NXApp delegate] respondsTo:aSelector])
        [[NXApp delegate] perform:aSelector with:self];
    return self;
}

- windowWillResize:sender toSize:(NXSize *)frameSize {
    NXRect mainRect, panelRect;

    [[NXApp mainWindow] getFrame:&mainRect];
    [panel getFrame:&panelRect];
    if (mainRect.size.width > panelRect.size.width) {
        if ((mainRect.size.width > (*frameSize).width) && 
            (mainRect.size.width <= (*frameSize).width + 20)) {
            (*frameSize).width = mainRect.size.width;
            (*frameSize).height = panelRect.size.height;
        }
    } else if (mainRect.size.width < panelRect.size.width) {
        if ((mainRect.size.width < (*frameSize).width) && 
            (mainRect.size.width >= (*frameSize).width - 20)) {
            (*frameSize).width = mainRect.size.width;
            (*frameSize).height = panelRect.size.height;
        }
    }

    return self;
}

@end
