#import "MP3Process.h"
#import <stdlib.h>
#import <string.h>
#import <signal.h>
#import <sys/wait.h>
#import <sys/time.h>       // for wait4()
#import <sys/resource.h>   // for wait4()

/* (by capri91) */
#ifdef m68k
#define PROC_NAME "nxmp3play"
#else
#define PROC_NAME "mpg123"
#endif

static char *_processPath;
static MP3Process *_sharedProcess = nil;

@interface MP3Process (Private)
- (int)pid;
- status:(int)aNumber;
@end

static void sig_child(int signal) {
    int pid = [_sharedProcess pid];
    int ret_pid;
    union wait status;

    if (pid)
        ret_pid = wait4(pid, &status, WNOHANG | WUNTRACED, NULL);
    else
        return;

    if ((ret_pid == pid) &&
        (WIFEXITED(status) || WIFSIGNALED(status))) {
        [_sharedProcess status:PS_STOPPED];
	if ([[_sharedProcess delegate] respondsTo:@selector(mp3ProcessEndedPlaying)]) {
            [[_sharedProcess delegate] mp3ProcessEndedPlaying];
            [NXApp getNextEvent:NX_NULLEVENTMASK waitFor:1 threshold:NX_BASETHRESHOLD];
        }
    }
}

@implementation MP3Process

+ initialize {
    _processPath = malloc(MAXPATHLEN);
    if (![[NXBundle mainBundle] getPath:_processPath forResource:PROC_NAME ofType:0]) {
/* (by capri91) */
/*         NXRunAlertPanel("Alert!", "Cannot find the nxmp3play executable file.\nCheck the application folder.", 0, 0, 0); */
        NXRunAlertPanel("Alert!", "Cannot find the "PROC_NAME" executable file.\nCheck the application folder.", 0, 0, 0);
        return nil;
    }
    if (access(_processPath, X_OK)) {
/* (by capri91) */
/*         NXRunAlertPanel("Alert!", "Cannot execute the nxmp3play.\nCheck the application folder.", 0, 0, 0); */
        NXRunAlertPanel("Alert!", "Cannot execute the "PROC_NAME".\nCheck the application folder.", 0, 0, 0);
        return nil;
    }

    signal(SIGCHLD, sig_child);
    return self;
}

- init {
    [super init];

    delegate = nil;
/* (by capri91) */
#ifdef m68k
    isLowSampleFrequency = YES;
#else
    isLowSampleFrequency = NO;
#endif
    isStereo = NO;
    qualityLevel = 0;
    pid = 0;

    return self;
}

- free {
    if ([self stop] == _sharedProcess)
        _sharedProcess = nil;
    return [super free];
}

+ (MP3Process *)sharedProcess {
    if (!_sharedProcess)
        _sharedProcess = [self new];
    return _sharedProcess;	
}

- delegate {
    return delegate;
}

- delegate:anObject {
    delegate = anObject;
    return self;
}

- (BOOL)isLowSampleFrequency {
    return isLowSampleFrequency;
}

- isLowSampleFrequency:(BOOL)yesNo {
    isLowSampleFrequency = yesNo;
    return self;
}

- (BOOL)isStereo {
    return isStereo;
}

- isStereo:(BOOL)yesNo {
    isStereo = yesNo;
    return self;
}

- (int)qualityLevel {
    return qualityLevel;
}

- qualityLevel:(int)aNumber {
    if ((aNumber >= 0) && (aNumber <= 2))
        qualityLevel = aNumber;
    return self;
}

- (int)status {
    return status;
}

#define ADD_ARG(arg) {\
    argv = realloc(argv, sizeof(char *) * (argc + 2));\
    *(argv + argc) = malloc(strlen(arg) + 1);\
    strcpy(*(argv + argc), arg);\
    argc++; }

- playFile:(const char *)file {
    if (!pid) {
        if ((pid = fork()) < 0) {
            printf("fork error!\n");
            return nil;
        }
        if (pid == 0) {
            int argc = 0;
            char **argv = malloc(sizeof(char *));
            char **argp;

            ADD_ARG(PROC_NAME);
/* (by capri91) */
#ifdef m68k
            ADD_ARG("-s");
#endif

            if (isLowSampleFrequency)
                ADD_ARG("-l");
            if (isStereo)
                ADD_ARG("-S");
            if (qualityLevel == 1)
                ADD_ARG("-1");
            if (qualityLevel == 2)
                ADD_ARG("-2");

            ADD_ARG(file);
            *(argv + argc) = NULL;

            close(0);
            close(1);
            close(2);
            execv(_processPath, argv);

            for (argp = argv; *argp; argp++)
                free(*argp);
            free(argv);

            exit(0);
        }

        [self status:PS_PLAYING];
    }
    return self;
}

- pause {
// signal handler disabled by shsuh 00.06.07
    if (pid) {
        signal(SIGCHLD, SIG_DFL);
        if (!kill(pid, SIGSTOP))
            [self status:PS_PAUSED];
        signal(SIGCHLD, sig_child);
    }
    return self;
}

- continue {
    if (pid && !kill(pid, SIGCONT))
        [self status:PS_PLAYING];
    return self;
}

- stop {
// signal handler disabled by shsuh 00.06.07
    if (pid) {
        signal(SIGCHLD, SIG_DFL);
        if (!kill(pid, SIGQUIT))
            [self status:PS_STOPPED];
        signal(SIGCHLD, sig_child);
    }
    return self;
}

@end

@implementation MP3Process (Private)

- (int)pid {
    return pid;
}

- status:(int)aNumber {
    if (status == aNumber)
        return self;

    switch (aNumber) {
        case PS_STOPPED:
            status = PS_STOPPED;
            pid = 0;
            break;
        case PS_PLAYING:
            status = PS_PLAYING;
            break;
        case PS_PAUSED:
            status = PS_PAUSED;
            break;
    }

    return self;
}

@end
