
#import "MainControl.h"
#import "PrefControl.h"
#import "Info.h"
#import "OptionManager.h"
#import "MP3Process.h"
#import "MP3List.h"
#import "MP3ListManager.h"
#import "MP3Browser.h"
#import "Common.h"

#define MAX_LIST	29

/* (by capri91) */
static const char *const mp3Type[3] = {"mp3", "MP3", NULL};

@implementation MainControl
- init {
    [super init];
    [[MP3Process sharedProcess] delegate:self];

    pref = [[PrefControl allocFromZone:[self zone]] init];
    optionMgr = [[OptionManager allocFromZone:[self zone]] init];
    listMgr = [[MP3ListManager allocFromZone:[self zone]] init];
	
    return self;
}

// Accessor methods
- pref {
    return pref;
}
- (const char *)currentListName {
	const char *listName = [playListButton title];
	if (listName && strlen(listName) > 0) return listName;	
	return NULL;	
}

// Info Menu 
- showPreference:sender {
    [pref orderFront:nil];
    return self;
}
- showInfo:sender {
    if(!info) info = [[Info allocFromZone:[self zone]] init];
    [info displayInfo:nil];

    return self;
}

// MP3 File & List
- openFile:sender {
    [listMgr addFile:sender];
    return self;
}
- openList:sender {
    [listMgr open:sender];
    return self;
}
- saveList:sender {
    [listMgr save:sender];
    return self;
}
- saveAsList:sender { // by hrjun 06.03
	[listMgr saveAs:sender];
	return self;
}

// Play with the processor
- play:sender {
    MP3Process *process = [MP3Process sharedProcess];

    switch ([process status]) {
        case PS_PLAYING:
//            isPlaying = NO;
            [process stop];
            [self startPlay];
            break;
        case PS_STOPPED:
            [listMgr reset];
            [self startPlay];
            break;
        case PS_PAUSED:
            [process continue];
            [self updateButtonStatus];
            break;
    }
    return self;
}
- pause:sender {
    [[MP3Process sharedProcess] pause];
//    [self updateButtonStatus]; // I think this does not need.(shsuh 00.06.07)
    return self;
}
- stop:sender {
//    isPlaying = NO;
    [[MP3Process sharedProcess] stop];
    [self updateButtonStatus];
    return self;
}
// by shsuh 00.05.31
- move:sender { /* sender is Matrix */
    MP3Process *process = [MP3Process sharedProcess];

    if ([[sender selectedCell] tag]) [listMgr next];
    else [listMgr previous];

    switch ([process status]) {
        case PS_PLAYING:
//            isPlaying = NO;
            [process stop];
            [self startPlay];
            break;
        case PS_PAUSED:
            [self stop:self];
        case PS_STOPPED:
            [self showCurrentFile];
            break;
    }
    return self;
}

// by shsuh 00.06.01
- setShuffle:sender {
    [listMgr isShuffle:[sender state]];
    return self;
}
- setRepeat:sender {
    [listMgr isRepeat:[sender state]];
    return self;
}

// MP3 File List
- displayFileList:sender {
    if ([sender state])
        [listMgr showPanel];
    else
        [listMgr hidePanel];

    return self;
}

// Play Option by hrjun 00.05.29
- displayOption:sender {
/* (by capri91) */
#ifdef m68k
    [optionMgr displayWindow:[sender state]];
#endif
    return self;
}

// Handle the list of mp3 files
- changePlayList:sender {
	const char *filename = [playListPopup selectedItem];
	if ([listMgr loadMP3List:filename]) [self stop:nil];

	return self;
}
- removePlayList:sender {
	return self;
}
- tryToAddPlayList:(const char*)name {
	Matrix *itemList = [playListPopup itemList];
	int count = [playListPopup count];
	
	if([self isContains:name inItemList:itemList]) return nil;
	if(count >= MAX_LIST) {
		[itemList setTitle:name at:MAX_LIST-limitCount :0];
		if(limitCount == MAX_LIST) limitCount = 0;
		else limitCount++;		
	} else {
		if(count == 0) [playListPopup addItem:name];
		else [playListPopup insertItem:name at:0];
	}
	[playListButton setTitle:name];
	[playListPopup sizeToFit];
	[playListPopup update];
	return self;
}
- (BOOL)isContains:(const char *)name inItemList:(Matrix *)itemList {
	int i, count = [[itemList cellList] count];
	id cell;
	
	for(i=0; i<count; i++) {
		cell = [[itemList cellList] objectAt:i];
		if(strcmp([cell title], name) == 0) return YES; // by hrjun 00.05.29
	}
	return NO;
}



// Internal methods
- checkStartupSound {
	if ([pref needStartupSound]) {
		const char *type;
		const char *filename;
		
		type = NXGetDefaultValue(PREF_VECTOR, "StartupFiletype");
		if (strcmp(type, "F") == 0) {
			filename = NXGetDefaultValue(PREF_VECTOR, "StartupMP3File");
			if (filename == NULL) return nil;
			else { // by hrjun 00.05.29
			    [[MP3Process sharedProcess] playFile:filename];
    			[self showMessage:filename fromPipe:NO];
			}
		} else {
			filename = NXGetDefaultValue(PREF_VECTOR, "StartupMP3List");
			if (filename == NULL) return nil;
			if ([listMgr loadMP3List:filename]) {
				[self tryToAddPlayList:filename];
				[self play:nil];
			}
		}
	}
	return self;
}
- checkAlarmSound {
	if ([pref needAlarmSound]) {
		// Add Timer
	}
	return self;
}
// by shsuh 00.05.31
/*- checkMP3ListAndPlay {
    id file = [listMgr currentFile];
    [listMgr follow];
    if (file != [listMgr currentFile])
        [self startPlay];
    else {
        isPlaying = NO;
// by hrjun 06.03        [self updateButtonStatus];
    }
    return self;
}*/
- startPlay {
    MP3Info *file = [listMgr currentFile];

    if (file) {
        char *path = malloc(strlen([file path]) + strlen([file filename]) + 2);
        sprintf(path, "%s/%s", [file path], [file filename]);
//        printf("### startPlay: %s\n", [file filename]); //***
        [[MP3Process sharedProcess] playFile:path];
        [self showCurrentFile];
        free(path);
//        isPlaying = YES;
    }
//    else
//        isPlaying = NO;

    [self updateButtonStatus];
    return self;
}
- showMessage:(const char*)str fromPipe:(BOOL)flag {
	[infoText setStringValue:str];
	return self;
}
- showCurrentFile {
    MP3Info *file = [listMgr currentFile];
    if (file)
        [self showMessage:[file filename] fromPipe:NO];
    return self;
}

- updateButtonStatus {
    switch ([[MP3Process sharedProcess] status]) {
        case PS_PLAYING:
            [playButtonMatrix selectCellAt:0 :1];
            break;
        case PS_PAUSED:
            [playButtonMatrix selectCellAt:0 :2];
            break;
        case PS_STOPPED:
            [playButtonMatrix selectCellAt:0 :0];
            break;
    }
    return self;
}

@end


@implementation MainControl (MP3ProcessDelegate)
- (void)mp3ProcessEndedPlaying {
    id file = [listMgr currentFile];

    [listMgr follow];
    if (file != [listMgr currentFile])
        [self startPlay];
    else
        [self updateButtonStatus];
}
/*- (void)mp3ProcessStartedPlaying { // What do you want to do here?
	[self updateButtonStatus];
}
- (void)mp3ProcessPausedPlaying { // What do you want to do here?
	[self updateButtonStatus];
}*/
@end

@implementation MainControl (ApplicationDelegate) 

- (BOOL)appAcceptsAnotherFile:sender // by hrjun 00.05.30
{
	return YES;
}
- (int)app:sender openFile:(const char*)filename type:(const char*)type { // by hrjun 00.05.30
	if (strcmp(type, DOC_EXTENSION) == 0) {
		if ([listMgr loadMP3List:filename]) {
			if([playListPopup count] > 0) [playListPopup removeItemAt:0];
			[playListButton setTitle:""];
			isFromWorkspace = YES;
			[self tryToAddPlayList:filename];
			[self play:nil];
			return YES;
		}
	}
	if (!strcmp(type, mp3Type[0]) || !strcmp(type, mp3Type[1])) {
		[[MP3Process sharedProcess] playFile:filename];
		[self showMessage:filename fromPipe:NO];
		isFromWorkspace = YES;
		return YES;
	}
	return NO;
}
- appDidInit:sender {
    [window setDelegate:self];
    [window setMiniwindowIcon:"Sound.tiff"];
    [window setMiniwindowTitle:"MP3Player"];
	
    [playListPopup setTarget:self];
    [playListPopup setAction:@selector(changePlayList:)];
    [playListPopup changeButtonTitle:YES];
    if (isFromWorkspace == NO) { // by hrjun 05.30
        if([playListPopup count] > 0) [playListPopup removeItemAt:0];
        [playListButton setTitle:""];
    }

    if (isFromWorkspace == NO) [self checkStartupSound];
    [self checkAlarmSound];

    return self;
}
- appWillTerminate:sender {
//    isPlaying = NO;
    [[MP3Process sharedProcess] free];
    [pref free];
    if (info) [info free];
    [listMgr free]; // by shsuh 00.05.31
    [window free];

    return self;
}
@end

@implementation MainControl (WindowDelegate)
// by shsuh 00.05.31
- panelWillClose:sender {
    if (sender == listMgr)
        [listPanelButton setState:![listPanelButton state]];
    else if (sender == optionMgr)
        [optionPanelButton setState:![optionPanelButton state]];
    return self;
}
- windowDidMove:sender {
    if (sender == window) {
        NXRect rect;
        [window getFrame:&rect];
        [listMgr movePanelTopLeftTo:rect.origin.x :rect.origin.y];
        [optionMgr movePanelTopLeftTo:rect.origin.x + rect.size.width :rect.origin.y + rect.size.height];
    }
    return self;
}
- windowWillMiniaturize:sender toMiniwindow:miniwindow { // by hrjun 06.03
	if (sender == window) {
		if (listMgr) [listMgr hidePanel];
		if (optionMgr) [[optionMgr optionWindow] orderOut:nil];
	}
	return self;
}
- windowDidDeminiaturize:sender { // by hrjun 06.03
	if (sender == window) {
		if ([listPanelButton state] == 1) [listMgr showPanel];
		if ([optionPanelButton state] == 1) [[optionMgr optionWindow] orderFront:nil];
	}
	return self;
}

@end


