
#import "OptionManager.h"
#import "MP3Process.h"

#define PLAY_OPTION "MP3PlayerOption"

@implementation OptionManager

+ initialize {
    static NXDefaultsVector optionVector = {
        {"Frequency", "LOW"},
        {"Output", "MONO"},
        {"Quality", "L0"},
        {NULL, NULL}
    };
    NXRegisterDefaults(PLAY_OPTION, optionVector);

    return self;
}
- init {
    [super init];
    [NXApp loadNibSection:"OptionManager.nib" owner:self];
    [self readVector];
    return self;
}
- readVector {
    const char *tmp = NULL;

    tmp = NXGetDefaultValue(PLAY_OPTION, "Frequency");
	if (strcmp(tmp, "HIGH") == 0) [frequencyMatrix selectCellAt:1 :0];
	else [qualityMatrix selectCellAt:0 :0];

    tmp = NXGetDefaultValue(PLAY_OPTION, "Output");
	if (strcmp(tmp, "STEREO") == 0) [outputMatrix selectCellWithTag:1];
	else [outputMatrix selectCellWithTag:0];

    tmp = NXGetDefaultValue(PLAY_OPTION, "Quality");
	if (strcmp(tmp, "L0") == 0) [qualityMatrix selectCellWithTag:0];
	else if (strcmp(tmp, "L1") == 0) [qualityMatrix selectCellWithTag:1];
	else if (strcmp(tmp, "L2") == 0) [qualityMatrix selectCellWithTag:2];

    return self;
}
- writeVector {
    char buf[5];
    static NXDefaultsVector optionVector = {
        {"Frequency", "LOW"},
        {"Output", "MONO"},
        {"Quality", "L0"},
        {NULL, NULL}
    };
	if ([frequencyMatrix selectedRow] == 0) optionVector[0].value = "LOW";
	else optionVector[0].value = "HIGH";
	
	if ([outputMatrix selectedRow] == 0) optionVector[1].value = "MONO";
	else optionVector[1].value = "STEREO";

    sprintf(buf, "L%d", [qualityMatrix selectedRow]);
    optionVector[2].value = buf;
	
    NXWriteDefaults(PLAY_OPTION, optionVector);
	NXUpdateDefaults();
    return self;
}
- optionWindow {
    return optionWindow;
}

- displayWindow:(BOOL)yesno {
    if (yesno) {
        [optionWindow orderFront:nil];
	    [self readVector];
    } else
        [optionWindow orderOut:nil];

    return self;
}
- changePlayOption {
    MP3Process *process = [MP3Process sharedProcess];

    [process isLowSampleFrequency:([frequencyMatrix selectedRow] ? NO : YES)];
    [process isStereo:([outputMatrix selectedRow] ? YES : NO)];
    [process qualityLevel:[qualityMatrix selectedRow]];

    return self;
}
- save:sender {
    [self writeVector];
    [self changePlayOption];
    return self;
}

- free {
    [optionWindow free];
    [super free];
    return self;
}

// by shsuh 00.06.02
- movePanelTopLeftTo:(NXCoord)x :(NXCoord)y {
    [optionWindow moveTopLeftTo:x :y];
    return self;
}
- windowWillClose:sender {
    SEL aSelector = @selector(panelWillClose:);
    if ([[NXApp delegate] respondsTo:aSelector])
        [[NXApp delegate] perform:aSelector with:self];
    return self;
}

@end
