
#import "PrefControl.h"
#import "Common.h"

/* (by capri91) */
static const char *const mp3Type[3] = {"mp3", "MP3", NULL};
static const char *const docType[2] = {DOC_EXTENSION, NULL};

typedef enum {FILE_TAG=0, LIST_TAG} _FiletypeTag;

@implementation PrefControl

+ initialize {
	static NXDefaultsVector newDefaults = {
		{"StatupUse", "NO"},
		{"StartupFiletype", "F"},
		{"StartupMP3File", ""},
		{"StartupMP3List", ""},
		{"AlarmUse", "NO"},
		{"AlarmRepeat", "NO"},
		{"AlarmFiletype", "F"},
		{"AlarmAMPM", "AM"},
		{"AlarmHour", "07"},
		{"AlarmMinute", "30"},
		{"AlarmMP3File", ""},
		{"AlarmMP3List", ""}, 
		{NULL, NULL}
	};
	NXRegisterDefaults(PREF_VECTOR, newDefaults);

	return self;
}
- readPreference {
	const char *tmp = NULL;
	
	tmp = NXGetDefaultValue(PREF_VECTOR, "StatupUse");
	if (tmp == NULL) [startupOption setState:0];
	else {
		if (strcmp(tmp, "YES") == 0) [startupOption setState:1];
		else [startupOption setState:0];
	}
	tmp = NXGetDefaultValue(PREF_VECTOR, "StartupFiletype");
	if (tmp == NULL) [startupFileMX selectCellAt:0 :0];
	else {
		if (strcmp(tmp, "F") == 0) [startupFiletype selectCellAt:0 :0];
		else [startupFiletype selectCellAt:1 :0];
	}
	tmp = NXGetDefaultValue(PREF_VECTOR, "StartupMP3File");
	[[startupFileMX cellAt:0 :0] setStringValue:tmp];
	tmp = NXGetDefaultValue(PREF_VECTOR, "StartupMP3List");
	[[startupFileMX cellAt:1 :0] setStringValue:tmp];
	
	tmp = NXGetDefaultValue(PREF_VECTOR, "AlarmUse");
	if (tmp == NULL) [[alarmCheck cellAt:0 :0] setState:0];
	else {
		if (strcmp(tmp, "YES") == 0) [[alarmCheck cellAt:0 :0] setState:1];
		else [[alarmCheck cellAt:0 :0] setState:0];
	}
	tmp = NXGetDefaultValue(PREF_VECTOR, "AlarmRepeat");
	if (tmp == NULL) [[alarmCheck cellAt:0 :0] setState:1];
	else {
		if (strcmp(tmp, "YES") == 0) [[alarmCheck cellAt:1 :0] setState:1];
		else [[alarmCheck cellAt:1 :0] setState:0];
	}
	tmp = NXGetDefaultValue(PREF_VECTOR, "AlarmFiletype");
	if (tmp == NULL) [alarmFileType selectCellAt:0 :0];
	else {
		if (strcmp(tmp, "F") == 0) [alarmFileType selectCellAt:0 :0];
		else [alarmFileType selectCellAt:1 :0];
	}
	tmp = NXGetDefaultValue(PREF_VECTOR, "AlarmAMPM");
	if (tmp == NULL) [[ampmOption selectCellAt:0 :0] setState:1];
	else {
		if (strcmp(tmp, "AM") == 0) [ampmOption selectCellAt:0 :0];
		else [ampmOption selectCellAt:0 :1];
	}
	tmp = NXGetDefaultValue(PREF_VECTOR, "AlarmHour");
	if (tmp == NULL) [hourText setStringValue:"07"];
	else [hourText setStringValue:tmp];
	tmp = NXGetDefaultValue(PREF_VECTOR, "AlarmMinute");
	if (tmp == NULL) [minuteText setStringValue:"30"];
	else [minuteText setStringValue:tmp];
	tmp = NXGetDefaultValue(PREF_VECTOR, "AlarmMP3File");
	[[alarmFileMX cellAt:0 :0] setStringValue:tmp];	tmp = NXGetDefaultValue(PREF_VECTOR, "AlarmMP3List");
	[[alarmFileMX cellAt:1 :0] setStringValue:tmp];	return self;
}
- writePreference {
	static NXDefaultsVector newDefaults = {
		{"StatupUse", "NO"},
		{"StartupFiletype", "F"},
		{"StartupMP3File", ""},
		{"StartupMP3List", ""},
		{"AlarmUse", "NO"},
		{"AlarmRepeat", "NO"},
		{"AlarmFiletype", "F"},
		{"AlarmAMPM", "AM"},
		{"AlarmHour", "07"},
		{"AlarmMinute", "30"},
		{"AlarmMP3File", ""},
		{"AlarmMP3List", ""}, 
		{NULL, NULL}
	};
	if ([startupOption state] == 1) newDefaults[0].value = "YES";
	else newDefaults[0].value = "NO";
	if ([[startupFiletype selectedCell] tag] == FILE_TAG) newDefaults[1].value = "F";
	else newDefaults[1].value = "L";
	newDefaults[2].value = (char*)[[startupFileMX cellAt:0 :0] stringValue];
	newDefaults[3].value = (char*)[[startupFileMX cellAt:1 :0] stringValue];
	
	if ([[alarmCheck cellAt:0 :0] state] == 1) newDefaults[4].value = "YES";
	else newDefaults[4].value = "NO";
	if ([[alarmCheck cellAt:1 :0] state] == 1) newDefaults[5].value = "YES";
	else newDefaults[5].value = "NO";
	if ([[alarmFileType selectedCell] tag] == FILE_TAG) newDefaults[6].value = "F";
	else newDefaults[6].value = "L";
	if ([[ampmOption selectedCell] tag] == 0) newDefaults[7].value = "AM";
	else newDefaults[7].value = "PM";
	newDefaults[8].value = (char*)[hourText stringValue];
	newDefaults[9].value = (char*)[minuteText stringValue];
	newDefaults[10].value = (char*)[[alarmFileMX cellAt:0 :0] stringValue];
	newDefaults[11].value = (char*)[[alarmFileMX cellAt:1 :0] stringValue];
	NXWriteDefaults(PREF_VECTOR, newDefaults);

	return self;
}

- (BOOL)needStartupSound {
	const char *tmp = NXGetDefaultValue(PREF_VECTOR, "StatupUse");
	if (tmp == NULL) return NO;
	else {
		if (strcmp(tmp, "YES") == 0) return YES;
	}
	return NO;
}
- (BOOL)needAlarmSound {
	const char *tmp = NXGetDefaultValue(PREF_VECTOR, "AlarmUse");
	if (tmp == NULL) return NO;
	else {
		if (strcmp(tmp, "YES") == 0) return YES;
	}
	return NO;
}

- init {
	[super init];
	[NXApp loadNibSection:"Preference.nib" owner:self];
	
	return self;
}
- orderFront:sender {
	if(![window isVisible]) [window center];
	[self readPreference];
	[window makeKeyAndOrderFront:nil];
	return self;
}

- revert:sender {
	[self readPreference];
	return self;
}
- save:sender {
	[self writePreference];
	return self;
}

- setStartupMusic:sender {
	_FiletypeTag tag;
	id open = [OpenPanel new];
	[open allowMultipleFiles:NO];
	[open chooseDirectories:NO];
	tag = [[sender selectedCell] tag];
/* (by capri91) */
/* 	const char *type; */
/* 	switch(tag) {
		case FILE_TAG:
			type = mp3Type; break;
		case LIST_TAG:
			type = docType; break;
	}
	if ([open runModalForTypes:type]) {
		[[alarmFileMX cellAt:tag :0] setStringValue:[open filename]];
	}
 */
	switch(tag) {
		case FILE_TAG:
			if ([open runModalForTypes:mp3Type]) {
				[[startupFileMX cellAt:tag :0] setStringValue:[open filename]];
			}
			break;
		case LIST_TAG:
			if ([open runModalForTypes:docType]) {
				[[startupFileMX cellAt:tag :0] setStringValue:[open filename]];
			}
			break;
	}

	return self;

}
- setAlarmMusic:sender {
	_FiletypeTag tag;
	id open = [OpenPanel new];
	[open allowMultipleFiles:NO];
	[open chooseDirectories:NO];
	tag = [[sender selectedCell] tag];
/* (by capri91) */
/* 	const char *type; */
/* 	switch(tag) {
		case FILE_TAG:
			type = mp3Type; break;
		case LIST_TAG:
			type = docType; break;
	}
	if ([open runModalForTypes:type]) {
		[[alarmFileMX cellAt:tag :0] setStringValue:[open filename]];
	}
 */
	switch(tag) {
		case FILE_TAG:
			if ([open runModalForTypes:mp3Type]) {
				[[startupFileMX cellAt:tag :0] setStringValue:[open filename]];
			}
			break;
		case LIST_TAG:
			if ([open runModalForTypes:docType]) {
				[[startupFileMX cellAt:tag :0] setStringValue:[open filename]];
			}
			break;
	}

	return self;
}

@end
